/*
 * (C) COPYRIGHT International Business Machines Corp. 2004,2005
 * All Rights Reserved
 * Licensed Materials - Property of IBM
 *
 * This sample program is provided AS IS and may be used, executed, copied
 * and modified without royalty payment by customer (a) for its own instruction and
 * study, (b) in order to develop applications designed to run with an IBM
 * WebSphere product, either for customer's own internal use or for
 * redistribution by customer, as part of such an application, in customer's
 * own products.
 *
 *  CEI/ws/code/cei.samples/src/EventService/com/ibm/events/samples/consumer/SampleEventConsumer.java, WAS.cei, WAS70.CEI, cf351435.01
 *  @Version 1.1 9/9/14
 *
 */
package com.ibm.events.samples.consumer;

import com.ibm.events.EventsException;
import com.ibm.events.access.EventAccess;
import com.ibm.events.access.EventAccessHome;

import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.ExtendedDataElement;

import javax.naming.Context;
import javax.naming.InitialContext;

import javax.rmi.PortableRemoteObject;

import java.rmi.RemoteException;

import java.util.Hashtable;
import java.util.List;


/**
 * This class provides sample code for an event consumer.
 */
public class SampleEventConsumer
{
    //~ Instance variables .........................................................................

    /** The event group to use for consuming events */
    private String eventGroup = "All events";

    /** An additional event selector for consuming events */
    private String eventSelector = null;

    /** The URL of the JNDI provider used to connect to the event infrastructure */
    private String providerUrl = "corbaloc:iiop:localhost:2809";

    /** Boolean used to decide whether to display the events */
    private boolean printEvents = false;

    /** The number of events to query from the database */
    private int number = 1;

    //~ Constructors ...............................................................................

    /**
     * Constructs the sample event consumer
     */
    public SampleEventConsumer()
    {
    }

    //~ Methods ....................................................................................

    /**
     * Sets the event group used for event consumption
     *
     * @param eventGroup The event group to use for event consumption
     */
    public void setEventGroup(String eventGroup)
    {
        this.eventGroup = eventGroup;
    }

    /**
     * Sets the event selector used for event consumption
     *
     * @param eventSelector The event selector to use for event consumption
     */
    public void setEventSelector(String eventSelector)
    {
        this.eventSelector = eventSelector;
    }

    //  ~ Methods ......................................................

    /**
     * Sets the number of events to query
     *
     * @param number The number of events to query
     */
    public void setNumber(int number)
    {
        this.number = number;
    }

    /**
     * Sets whether events should be printed or not
     *
     * @param printEvents Whether to print out events or not
     */
    public void setPrintEvents(boolean printEvents)
    {
        this.printEvents = printEvents;
    }

    /**
     * Sets the JNDI URL provider.
     *
     * @param providerUrl The JNDI URL provider
     */
    public void setProviderUrl(String providerUrl)
    {
        this.providerUrl = providerUrl;
    }

    /**
     * Displays the command-line usage for this sample.
     */
    public void displayUsage()
    {
        System.out.println("SampleEventConsumer [-j <providerUrl>] [-g <eventGroup>] [-s <eventSelector>] [-n <number>] [-e]");
        System.out.println("\t-j <providerUrl> : The JNDI provider URL. Such as corbaloc:iiop:localhost:2809");
        System.out.println("\t-g <eventGroup> : The event group used for the query");
        System.out.println("\t-s <eventSelector> : The event selector used for the query");
        System.out.println("\t-n <number> : The number of events to be returned");
        System.out.println("\t-e : To print the queried events");
    }

    /**
     * Main method that will take command line arguments and run the event consumer
     *
     * @param args Array arguements passed form the command line
     */
    public static void main(String args[])
    {
        // create an instance of the sample event consumer
        SampleEventConsumer eventConsumer = new SampleEventConsumer();

        // process the command line args
        for(int i = 0; i < args.length; i++)
        {
            if(args[i].equals("-j"))
            {
                i++;

                if(i < args.length)
                {
                    eventConsumer.setProviderUrl(args[i]);
                }
            }
            else if(args[i].equals("-g"))
            {
                i++;

                if(i < args.length)
                {
                    eventConsumer.setEventGroup(args[i]);
                }
            }
            else if(args[i].equals("-n"))
            {
                i++;

                if(i < args.length)
                {
                    eventConsumer.setNumber(Integer.parseInt(args[i]));
                }
            }
            else if(args[i].equals("-s"))
            {
                i++;

                if(i < args.length)
                {
                    eventConsumer.setEventSelector(args[i]);
                }
            }
            else if(args[i].equals("-e"))
            {
                eventConsumer.setPrintEvents(true);
            }
            else
            {
                eventConsumer.displayUsage();
                System.exit(1);
            }
        }

        // run the event consumer
        eventConsumer.run();

        System.exit(0);
    }

    /**
     * Runs the sample event consumer
     */
    public void run()
    {
        try
        {
            // get the time that the consumer started
            long startTime = System.currentTimeMillis();

            // variable to hold the events returned from the query
            CommonBaseEvent events[]   = null;

            // look up the event access enterprise bean
            Hashtable jndiConfig = new Hashtable();
            jndiConfig.put(Context.INITIAL_CONTEXT_FACTORY,
                           "com.ibm.websphere.naming.WsnInitialContextFactory");
            jndiConfig.put(Context.PROVIDER_URL,
                           providerUrl);

            Context         initial = new InitialContext(jndiConfig);

            Object          objref = initial.lookup("ejb/com/ibm/events/access/EventAccess");

            EventAccessHome eventAccessHome =
                (EventAccessHome) PortableRemoteObject.narrow(objref,
                                                              EventAccessHome.class);

            EventAccess eventAccess = eventAccessHome.create();

            if(number > 0)
            {
                // query just the specified number of events
                events =
                    eventAccess.queryEventsByEventGroup(eventGroup,
                                                        eventSelector,
                                                        true,
                                                        number);
            }
            else
            {
                // query all matching events
                events = eventAccess.queryEventsByEventGroup(eventGroup,
                                                             eventSelector,
                                                             true);
            }

            // set the time that the query finshed
            long endTime = System.currentTimeMillis();

            // display the number of events returned from the query
            // and the time that it took to query the events
            System.out.println(events.length + " Events returned from Event Access in " +
                               (endTime - startTime) + " millisecond");

            // if printEvents is enabled print out the events returned from the query
            if(printEvents)
            {
                for(int j = 0; j < events.length; j++)
                {
                    System.out.println("===============================================================================");
                    System.out.println(events[j]);
                }
            }
        }
        catch(RemoteException e)
        {
            e.printStackTrace();
        }
        catch(EventsException e)
        {
            e.printStackTrace();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }
}