/*
 * (C) COPYRIGHT International Business Machines Corp. 2004,2005
 * All Rights Reserved
 * Licensed Materials - Property of IBM
 *
 * This sample program is provided AS IS and may be used, executed, copied
 * and modified without royalty payment by customer (a) for its own instruction and
 * study, (b) in order to develop applications designed to run with an IBM
 * WebSphere product, either for customer's own internal use or for
 * redistribution by customer, as part of such an application, in customer's
 * own products.
 *
 *  CEI/ws/code/cei.samples/src/EventService/com/ibm/events/samples/subscriber/SampleEventSubscriber.java, WAS.cei, WAS70.CEI, cf351435.01
 *  @Version 1.1 9/9/14
 *
 */
package com.ibm.events.samples.subscriber;

import com.ibm.events.EventsException;
import com.ibm.events.notification.EventNotification;
import com.ibm.events.notification.MessagePort;
import com.ibm.events.notification.NotificationHelper;
import com.ibm.events.notification.NotificationHelperFactory;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;

import javax.naming.Context;
import javax.naming.InitialContext;

import java.util.Hashtable;


/**
 * This class provides sample code for an event subscriber.
 */
public class SampleEventSubscriber
{
    //~ Instance variables .........................................................................

    /** Reference to the notification helper */
    private NotificationHelper notificationHelper = null;

    /** Reference to the JMS queue */
    private Queue queue = null;

    /** Reference to the JMS queue connection */
    private QueueConnection queueConnection = null;

    /** Reference to the JMS queue connection factory */
    private QueueConnectionFactory queueFactory = null;

    /** Reference to the JMS queue receiver */
    private QueueReceiver queueReceiver = null;

    /** Reference to the JMS queue session */
    private QueueSession queueSession = null;

    /** The event group to use for consuming events */
    private String eventGroup = "All events";

    /** An additional event selector for consuming events */
    private String eventSelector = null;

    /** The URL of the JNDI provider used to connect to the event infrastructure */
    private String providerUrl = "corbaloc:iiop:localhost:2809";

    /** Reference to the JMS topic */
    private Topic topic = null;

    /** Reference to the JMS topic connection */
    private TopicConnection topicConnection = null;

    /** Reference to the JMS topic connection factory */
    private TopicConnectionFactory topicFactory = null;

    /** Reference to the JMS topic session */
    private TopicSession topicSession = null;

    /** Reference to the JMS topic subscriber */
    private TopicSubscriber topicSubscriber = null;

    /** Boolean used to decided whether to display the events */
    private boolean printEvents = false;

    /** The index of which distrbution queue to listen on */
    private int index = 0;

    /** The number of events to receiver from the subscription */
    private int number = 1;

    //~ Constructors ...............................................................................

    /**
     * Construct the sample event subscriber
     */
    public SampleEventSubscriber()
    {
    }

    //~ Methods ....................................................................................

    /**
     * Sets the event group used for event subscription
     *
     * @param eventGroup The event group to use for event subscription
     */
    public void setEventGroup(String eventGroup)
    {
        this.eventGroup = eventGroup;
    }

    /**
     * Sets the event selector used for event subscription
     *
     * @param eventSelector The event selector to use for event subscription
     */
    public void setEventSelector(String eventSelector)
    {
        this.eventSelector = eventSelector;
    }

    /**
     * Set the distribution queue index
     *
     * @param index The index of the distribution queue to listen on
     */
    public void setIndex(int index)
    {
        this.index = index;
    }

    /**
     * Set the number of events to receive from subscription
     *
     * @param number The number of events to receive from subscription
     */
    public void setNumber(int number)
    {
        this.number = number;
    }

    /**
     * Set whether events should be printed or not
     *
     * @param printEvents Whether to print out events or not
     */
    public void setPrintEvents(boolean printEvents)
    {
        this.printEvents = printEvents;
    }

    /**
     * Set the JNDI URL provider.
     *
     * @param providerUrl The JNDI URL provider
     */
    public void setProviderUrl(String providerUrl)
    {
        this.providerUrl = providerUrl;
    }

    /**
     * Display the command line usage for this sample.
     */
    public void displayUsage()
    {
        System.out.println("SampleEventSubscriber [-j <providerUrl>] [-g <eventGroup>] [-s <eventSelector>] [-n <number>] [-q <index>] [-e]");
        System.out.println("\t-j <providerUrl> : The JNDI provider URL. Such as corbaloc:iiop:localhost:2809");
        System.out.println("\t-g <eventGroup> : The event group used for subscription.");
        System.out.println("\t-s <eventSelector> : The event selector used for filtering notifcations.");
        System.out.println("\t-n <number> : The number of events to return");
        System.out.println("\t-e : To print the events returned");
        System.out.println("\t-q <index>: To use the distribution queue at the specified index instead of the topic");
    }

    /**
     * Main method that will take command line arguments and run the event subscriber
     *
     * @param args Array arguments passed form the command line
     *
     * @throws Exception If there is an error processing commands
     */
    public static void main(String args[]) throws Exception
    {
        // create an instance of the sample event subscriber
        SampleEventSubscriber eventSubscriber = new SampleEventSubscriber();

        // process the command line args
        for(int i = 0; i < args.length; i++)
        {
            if(args[i].equals("-j"))
            {
                i++;

                if(i < args.length)
                {
                    eventSubscriber.setProviderUrl(args[i]);
                }
            }
            else if(args[i].equals("-g"))
            {
                i++;

                if(i < args.length)
                {
                    eventSubscriber.setEventGroup(args[i]);
                }
            }
            else if(args[i].equals("-s"))
            {
                i++;

                if(i < args.length)
                {
                    eventSubscriber.setEventSelector(args[i]);
                }
            }
            else if(args[i].equals("-n"))
            {
                i++;

                if(i < args.length)
                {
                    eventSubscriber.setNumber(Integer.parseInt(args[i]));
                }
            }
            else if(args[i].equals("-q"))
            {
                i++;

                if(i < args.length)
                {
                    eventSubscriber.setIndex(Integer.parseInt(args[i]));
                }
            }
            else if(args[i].equals("-e"))
            {
                eventSubscriber.setPrintEvents(true);
            }
            else
            {
                eventSubscriber.displayUsage();
                System.exit(1);
            }
        }

        // run the event subscriber
        eventSubscriber.run();

        System.exit(0);
    }

    /**
     * Run the sample event subscriber
     */
    public void run()
    {
        try
        {
            // get the time that the consumer started
            long startTime = System.currentTimeMillis();

            // create the JNDI context
            Hashtable jndiConfig = new Hashtable();
            jndiConfig.put(Context.INITIAL_CONTEXT_FACTORY,
                           "com.ibm.websphere.naming.WsnInitialContextFactory");
            jndiConfig.put(Context.PROVIDER_URL,
                           providerUrl);

            Context jndiContext = new InitialContext(jndiConfig);

            // create the notification helper
            NotificationHelperFactory nhFactory =
                (NotificationHelperFactory) jndiContext.lookup("com/ibm/events/NotificationHelperFactory");
            notificationHelper = nhFactory.getNotificationHelper();

            // get the topic for the event group
            if(eventGroup != null)
            {
                // if queue index is 0 then create a topic connection
                if(index == 0)
                {
                    createTopicSubscriber(jndiContext,
                                          notificationHelper);
                }
                else
                {
                    createQueueReceiver(jndiContext,
                                        notificationHelper);
                }

                // loop to receive events
                for(int i = 0; i < number; i++)
                {
                    Message eventMsg = null;

                    if(index == 0)
                    {
                        eventMsg = topicSubscriber.receive();
                    }
                    else
                    {
                        eventMsg = queueReceiver.receive();
                    }

                    // get the event from the message
                    EventNotification eventNotifications[] =
                        notificationHelper.getEventNotifications(eventMsg);

                    for(int j = 0; j < eventNotifications.length; j++)
                    {
                        if(printEvents)
                        {
                            System.out.println("===============================================================================");

                            if(eventNotifications[j].getNotificationType() == NotificationHelper.REMOVE_EVENT_NOTIFICATION_TYPE)
                            {
                                System.out.println("Notification Type = REMOVE_EVENT_NOTIFICATION_TYPE");
                                System.out.println("GlobalInstanceId = " +
                                                   eventNotifications[j].getGlobalInstanceId());
                            }
                            else if(eventNotifications[j].getNotificationType() == NotificationHelper.UPDATE_EVENT_NOTIFICATION_TYPE)
                            {
                                System.out.println("Notification Type = UPDATE_EVENT_NOTIFICATION_TYPE");
                                System.out.println("Event = " + eventNotifications[j].getEvent());
                            }
                            else
                            {
                                System.out.println("Notification Type = CREATE_EVENT_NOTIFICATION_TYPE");
                                System.out.println("Event = " + eventNotifications[j].getEvent());
                            }
                        }
                    }
                }
            }
            else
            {
                System.out.println("No topic found in JNDI for event group " + eventGroup);

                System.exit(1);
            }

            // set the time that the subscription finshed
            long endTime = System.currentTimeMillis();

            // display the number of events returned from the subscription
            // and the time that it took to receive the events
            System.out.println(number + " Events received from Notification Helper in " +
                               (endTime - startTime) + " millisecond");
        }
        catch(EventsException e)
        {
            e.printStackTrace();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Create a queue receiver to listen for event notifications
     *
     * @param jndiContext The JNDI context used to look up the JMS destination
     * @param notificationHelper An instance of the notifcation helper
     *
     * @throws Exception If there is an error creating the receiver
     */
    private void createQueueReceiver(Context            jndiContext,
                                     NotificationHelper notificationHelper)
                              throws Exception
    {
        MessagePort msgPorts[] = notificationHelper.getJmsQueues(eventGroup);

        if(index > msgPorts.length)
        {
            System.out.println("There is not a JMS queue at the specified index " + index);

            System.exit(1);
        }
        else
        {
            String queueJndiName        = msgPorts[index - 1].getDestinationJndiName();
            String queueFactoryJndiName = msgPorts[index - 1].getConnectionFactoryJndiName();

            // lookup the queue and queue connection factory
            if((queueJndiName != null) && (queueFactoryJndiName != null))
            {
                System.out.println("Connection to QueueConnectionFactory : " +
                                   queueFactoryJndiName + " Queue: " + queueJndiName);
                queue            = (Queue) jndiContext.lookup(queueJndiName);
                queueFactory     = (QueueConnectionFactory) jndiContext.lookup(queueFactoryJndiName);

                if((queue != null) && (queueFactory != null))
                {
                    // create topic connection objects
                    queueConnection     = queueFactory.createQueueConnection();
                    queueSession =
                        queueConnection.createQueueSession(false,
                                                           Session.AUTO_ACKNOWLEDGE);
                    queueReceiver = queueSession.createReceiver(queue);

                    // start the queue connection
                    queueConnection.start();
                }
                else
                {
                    System.out.println("No event group defined named " + eventGroup);

                    System.exit(1);
                }
            }
            else
            {
                System.out.println("No queue defined for event group " + eventGroup);

                System.exit(1);
            }
        }
    }

    /**
     * Create a topic subscriber to listen for event notifications
     *
     * @param jndiContext The JNDI context used to look up the JMS destination
     * @param notificationHelper An instance of the notifcation helper
     *
     * @throws Exception If there is an error creating the subscriber
     */
    private void createTopicSubscriber(Context            jndiContext,
                                       NotificationHelper notificationHelper)
                                throws Exception
    {
        MessagePort msgPort              = notificationHelper.getJmsTopic(eventGroup);
        String      topicJndiName        = msgPort.getDestinationJndiName();
        String      topicFactoryJndiName = msgPort.getConnectionFactoryJndiName();

        // lookup the topic and topic connection factory
        if((topicJndiName != null) && (topicFactoryJndiName != null))
        {
            System.out.println("Connection to TopicConnectionFactory : " + topicFactoryJndiName +
                               " Topic: " + topicJndiName);
            topic            = (Topic) jndiContext.lookup(topicJndiName);
            topicFactory     = (TopicConnectionFactory) jndiContext.lookup(topicFactoryJndiName);

            if((topic != null) && (topicFactory != null))
            {
                // create topic connection objects
                topicConnection     = topicFactory.createTopicConnection();
                topicSession        = topicConnection.createTopicSession(false,
                                                                         Session.AUTO_ACKNOWLEDGE);
                topicSubscriber = topicSession.createSubscriber(topic);

                // start the topic connection
                topicConnection.start();
            }
            else
            {
                System.out.println("No event group defined named " + eventGroup);

                System.exit(1);
            }
        }
        else
        {
            System.out.println("No topic defined for event group " + eventGroup);

            System.exit(1);
        }
    }
}